#!/usr/bin/env python

import threading

from dm.common.utility.loggingManager import LoggingManager

class FileProcessingThread(threading.Thread):

    def __init__ (self, name, eventFlag, fileProcessorDict, fileProcessorKeyList, fileProcessingQueue, processedFileDict, unprocessedFileDict):
    
        threading.Thread.__init__(self)
        self.setName(name)
        self.exitFlag = False
        self.eventFlag = eventFlag
        self.fileProcessorDict = fileProcessorDict
        self.fileProcessorKeyList = fileProcessorKeyList
        self.fileProcessingQueue = fileProcessingQueue
        self.processedFileDict = processedFileDict 
        self.unprocessedFileDict = unprocessedFileDict 
        self.logger = LoggingManager.getInstance().getLogger(name)

    def run(self):
        self.logger.debug('Starting thread: %s' % self.getName())
        while True:
            if self.exitFlag:
                self.logger.debug('Exit flag set, %s done' % self.getName())
                break

            while True:
                file = self.fileProcessingQueue.pop()
                if file is None:
                    break
                filePath = file.getFilePath()
                daqPath = file.getDaqPath()
                experiment = file.getExperiment()
                
                try:
                    for processorKey in self.fileProcessorKeyList: 
                        processor = self.fileProcessorDict.get(processorKey)
                        processorName = processor.__class__.__name__
                        fileProcessedByDict = file.get('processedByDict', {})
                        if fileProcessedByDict.has_key(processorName):
                            self.logger.debug('%s has already been processed by %s ' % (file, processorName))
                            continue

                        self.logger.debug('%s is about to process file %s ' % (processorName, file))
                        try:
                            processor.processFile(filePath, daqPath, experiment)
                            fileProcessedByDict[processorName] = True
                            file['processedByDict'] = fileProcessedByDict
                            self.logger.debug('%s processed file %s ' % (processorName, file))
                        except Exception, ex:
                            self.logger.exception(ex)
                            self.logger.debug('%s processing failed for file %s ' % (processorName, file))
                            fileProcessingDict = file.get('processingDict', {})
                            file['processingDict'] = fileProcessingDict
                            processorDict = fileProcessingDict.get(processorName, {}) 
                            processorDict['lastError'] = ex
                            nRetriesLeft = processorDict.get('numberOfRetriesLeft', processor.getNumberOfRetries()) 
                            self.logger.debug('Number of %s retries left for file %s: %s' % (processorName, file, nRetriesLeft))
                            processorDict['numberOfRetriesLeft'] = nRetriesLeft - 1
                            if nRetriesLeft <= 0:
                                self.logger.debug('No more %s retries left for file %s' % (processorName, file))
                                self.unprocessedFileDict[filePath] = file
                            else:
                                retryWaitPeriod = processor.getRetryWaitPeriodInSeconds()
                                self.logger.debug('%s will retry processing file %s in %s seconds' % (processorName, file, retryWaitPeriod))
                                self.fileProcessingQueue.push(file, retryWaitPeriod)

                except Exception, ex:
                    self.logger.exception(ex)

            self.eventFlag.wait()

    def stop(self):
        self.exitFlag = True

####################################################################
# Testing

if __name__ == '__main__':
    pass


